@@ -10,6 +10,8 @@ import importlib
 from .compat import OrderedDict
 from abc import ABCMeta, abstractmethod
 
+from django.http import HttpRequest
+from django.contrib.auth.models import AnonymousUser
 from django.contrib.admindocs.utils import trim_docstring
 from django.utils.encoding import smart_text
 
@@ -18,6 +20,7 @@ from rest_framework.views import get_view_name, get_view_description
 from rest_framework import viewsets
 from rest_framework.compat import apply_markdown
 from rest_framework.utils import formatting
+from django.utils import six
 
 
 def get_default_value(field):
@@ -117,15 +120,6 @@ class BaseViewIntrospector(object):
     def get_iterator(self):
         return self.__iter__()
 
-    def get_serializer_class(self):
-        if hasattr(self.callback, 'get_serializer_class'):
-            view = self.callback()
-            if not hasattr(view, 'kwargs'):
-                view.kwargs = dict()
-            if hasattr(self.pattern, 'default_args'):
-                view.kwargs.update(self.pattern.default_args)
-            return view.get_serializer_class()
-
     def get_description(self):
         """
         Returns the first sentence of the first line of the class docstring
@@ -175,11 +169,31 @@ class BaseMethodIntrospector(object):
         parser.object = new_object
         return parser
 
+    def ask_for_serializer_class(self):
+        if hasattr(self.callback, 'get_serializer_class'):
+            view = self.create_view()
+            parser = self.get_yaml_parser()
+            mock_view = parser.get_view_mocker(self.callback)
+            view = mock_view(view)
+            if view is not None:
+                return view.get_serializer_class()
+
+    def create_view(self):
+        view = self.callback()
+        if not hasattr(view, 'kwargs'):
+            view.kwargs = dict()
+        if hasattr(self.parent.pattern, 'default_args'):
+            view.kwargs.update(self.parent.pattern.default_args)
+        view.request = HttpRequest()
+        view.request.user = AnonymousUser()
+        view.request.method = self.method
+        return view
+
     def get_serializer_class(self):
         parser = self.get_yaml_parser()
         serializer = parser.get_serializer_class(self.callback)
         if serializer is None:
-            serializer = self.parent.get_serializer_class()
+            serializer = self.ask_for_serializer_class()
         return serializer
 
     def get_response_serializer_class(self):
@@ -551,6 +565,13 @@ class ViewSetMethodIntrospector(BaseMethodIntrospector):
         """
         return self.retrieve_docstring()
 
+    def create_view(self):
+        view = super(ViewSetMethodIntrospector, self).create_view()
+        if not hasattr(view, 'action'):
+            setattr(view, 'action', self.method)
+        view.request.method = self.http_method
+        return view
+
 
 def multi_getattr(obj, attr, default=None):
     """
@@ -818,6 +839,8 @@ class YAMLDocstringParser(object):
         Retrieves response serializer class from YAML object
         """
         serializer = self.object.get('response_serializer', None)
+        if isinstance(serializer, list):
+            serializer = serializer[0]
         try:
             return self._load_class(serializer, callback)
         except (ImportError, ValueError):
@@ -844,6 +867,12 @@ class YAMLDocstringParser(object):
             })
         return messages
 
+    def get_view_mocker(self, callback):
+        view_mocker = self.object.get('view_mocker', lambda a: a)
+        if isinstance(view_mocker, six.string_types):
+            view_mocker = self._load_class(view_mocker, callback)
+        return view_mocker
+
     def get_parameters(self):
         """
         Retrieves parameters from YAML object
